import warning from 'warning';
import { isVue2, defineComponent, getCurrentInstance, ref, onMounted, onUpdated, h } from 'vue-demi';
import { debug } from '../env';
import { COMPONENT_NAME } from '../shared/context';
import { pushSuspenseInstance, popSuspenseInstance } from '../shared/suspenseInstance';
import { FragmentComponent as Fragment } from '../shared/fragment';
export var del = function (asyncFactory, error) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys;
    if (error) {
        suspIns.reject(error);
        return;
    }
    asyncFactorys.delete(asyncFactory);
    if (asyncFactorys.size === 0) {
        suspIns.resolve();
    }
};
export var add = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys || (suspIns.asyncFactorys = new Set());
    asyncFactorys.add(asyncFactory);
    suspIns.fallback();
};
export var has = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return warning(!debug, 'No Suspense instance');
    }
    return suspIns.asyncFactorys && suspIns.asyncFactorys.has(asyncFactory);
};
export var suspenseOptions = {
    mode: 'visible',
    hiddleWrapperTag: 'div',
};
export function setSuspenseOptions(options) {
    Object.assign(suspenseOptions, options);
}
/**
 * Suspense component,
 * Vue2 compatible
 */
export var Suspense = defineComponent({
    name: COMPONENT_NAME,
    props: {
        timeout: {
            type: Number,
            default: 0,
        },
    },
    emits: ['pending', 'resolve', 'fallback'],
    setup: function (props, _a) {
        var slots = _a.slots, emit = _a.emit;
        var resolved = ref(false);
        var rejected = ref(false);
        var displayFallback = ref(false);
        var _time;
        var setupFallback = function () {
            if (resolved.value)
                return;
            if (!displayFallback.value) {
                emit('pending');
                if (props.timeout > 0) {
                    _time = setTimeout(function () {
                        displayFallback.value = true;
                        emit('fallback');
                    }, props.timeout);
                }
                else if (props.timeout === 0) {
                    displayFallback.value = true;
                    emit('fallback');
                }
            }
        };
        var destoryFallback = function () {
            if (_time) {
                clearTimeout(_time);
                _time = null;
            }
            displayFallback.value = false;
        };
        var onResolve = function () {
            destoryFallback();
            emit('resolve');
            resolved.value = true;
        };
        var onReject = function (err) {
            destoryFallback();
            emit('fallback');
            rejected.value = true;
        };
        var currPublicInstance = getCurrentInstance().proxy;
        currPublicInstance.resolve = onResolve;
        currPublicInstance.reject = onReject;
        currPublicInstance.fallback = setupFallback;
        pushSuspenseInstance(currPublicInstance);
        onMounted(function () {
            if (!currPublicInstance.asyncFactorys) {
                onResolve();
            }
        });
        onUpdated(function () {
            if (!resolved.value)
                return;
            popSuspenseInstance();
        });
        return {
            resolved: resolved,
            rejected: rejected,
            displayFallback: displayFallback,
            createWrapper: createWrapper,
            createHiddenWrapper: createHiddenWrapper,
        };
    },
    render: function () {
        var _a, _b, _c, _d;
        var isVisible = suspenseOptions.mode === 'visible';
        // @ts-expect-error type error
        // FIXME: warn in Vue 3 because of import problem
        var emptyNode = isVue2 ? this._e('suspense') : h(null, 'suspense');
        var fallback = this.displayFallback ? ((_b = (_a = this.$slots).fallback) === null || _b === void 0 ? void 0 : _b.call(_a)) || (debug ? [emptyNode] : []) : [];
        // The `children` is the real content to be rendered
        var children = ((_d = (_c = this.$slots).default) === null || _d === void 0 ? void 0 : _d.call(_c)) || (debug ? [emptyNode] : []);
        var rendered;
        if (this.rejected && this.$slots.error) {
            rendered = createWrapper(this.$slots.error);
        }
        else {
            rendered = isVisible
                ? createWrapper(this.resolved ? children : children.concat(fallback))
                : createWrapper([
                    // We need to render the children, but we should not show the rendered content.
                    createHiddenWrapper(children, this.resolved),
                    fallback,
                ]);
        }
        return rendered;
    },
});
function createWrapper(children) {
    if (!isVue2 && typeof children !== 'function') {
        children = function () { return children; };
    }
    return h(Fragment, children);
}
function createHiddenWrapper(children, display) {
    if (!isVue2 && typeof children !== 'function') {
        children = function () { return children; };
    }
    return h(suspenseOptions.hiddleWrapperTag, {
        style: { display: display ? 'block' : 'none' },
        class: { 'suspense-hidden-wrapper': true },
    }, children);
}
