"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Suspense = exports.setSuspenseOptions = exports.suspenseOptions = exports.has = exports.add = exports.del = void 0;
var warning_1 = __importDefault(require("warning"));
var vue_demi_1 = require("vue-demi");
var env_1 = require("../env");
var context_1 = require("../shared/context");
var suspenseInstance_1 = require("../shared/suspenseInstance");
var fragment_1 = require("../shared/fragment");
var del = function (asyncFactory, error) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return (0, warning_1.default)(!env_1.debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys;
    if (error) {
        suspIns.reject(error);
        return;
    }
    asyncFactorys.delete(asyncFactory);
    if (asyncFactorys.size === 0) {
        suspIns.resolve();
    }
};
exports.del = del;
var add = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return (0, warning_1.default)(!env_1.debug, 'No Suspense instance');
    }
    var asyncFactorys = suspIns.asyncFactorys || (suspIns.asyncFactorys = new Set());
    asyncFactorys.add(asyncFactory);
    suspIns.fallback();
};
exports.add = add;
var has = function (asyncFactory) {
    var suspIns = asyncFactory.suspenseInstance;
    if (!suspIns) {
        return (0, warning_1.default)(!env_1.debug, 'No Suspense instance');
    }
    return suspIns.asyncFactorys && suspIns.asyncFactorys.has(asyncFactory);
};
exports.has = has;
exports.suspenseOptions = {
    mode: 'visible',
    hiddleWrapperTag: 'div',
};
function setSuspenseOptions(options) {
    Object.assign(exports.suspenseOptions, options);
}
exports.setSuspenseOptions = setSuspenseOptions;
/**
 * Suspense component,
 * Vue2 compatible
 */
exports.Suspense = (0, vue_demi_1.defineComponent)({
    name: context_1.COMPONENT_NAME,
    props: {
        timeout: {
            type: Number,
            default: 0,
        },
    },
    emits: ['pending', 'resolve', 'fallback'],
    setup: function (props, _a) {
        var slots = _a.slots, emit = _a.emit;
        var resolved = (0, vue_demi_1.ref)(false);
        var rejected = (0, vue_demi_1.ref)(false);
        var displayFallback = (0, vue_demi_1.ref)(false);
        var _time;
        var setupFallback = function () {
            if (resolved.value)
                return;
            if (!displayFallback.value) {
                emit('pending');
                if (props.timeout > 0) {
                    _time = setTimeout(function () {
                        displayFallback.value = true;
                        emit('fallback');
                    }, props.timeout);
                }
                else if (props.timeout === 0) {
                    displayFallback.value = true;
                    emit('fallback');
                }
            }
        };
        var destoryFallback = function () {
            if (_time) {
                clearTimeout(_time);
                _time = null;
            }
            displayFallback.value = false;
        };
        var onResolve = function () {
            destoryFallback();
            emit('resolve');
            resolved.value = true;
        };
        var onReject = function (err) {
            destoryFallback();
            emit('fallback');
            rejected.value = true;
        };
        var currPublicInstance = (0, vue_demi_1.getCurrentInstance)().proxy;
        currPublicInstance.resolve = onResolve;
        currPublicInstance.reject = onReject;
        currPublicInstance.fallback = setupFallback;
        (0, suspenseInstance_1.pushSuspenseInstance)(currPublicInstance);
        (0, vue_demi_1.onMounted)(function () {
            if (!currPublicInstance.asyncFactorys) {
                onResolve();
            }
        });
        (0, vue_demi_1.onUpdated)(function () {
            if (!resolved.value)
                return;
            (0, suspenseInstance_1.popSuspenseInstance)();
        });
        return {
            resolved: resolved,
            rejected: rejected,
            displayFallback: displayFallback,
            createWrapper: createWrapper,
            createHiddenWrapper: createHiddenWrapper,
        };
    },
    render: function () {
        var _a, _b, _c, _d;
        var isVisible = exports.suspenseOptions.mode === 'visible';
        // @ts-expect-error type error
        // FIXME: warn in Vue 3 because of import problem
        var emptyNode = vue_demi_1.isVue2 ? this._e('suspense') : (0, vue_demi_1.h)(null, 'suspense');
        var fallback = this.displayFallback ? ((_b = (_a = this.$slots).fallback) === null || _b === void 0 ? void 0 : _b.call(_a)) || (env_1.debug ? [emptyNode] : []) : [];
        // The `children` is the real content to be rendered
        var children = ((_d = (_c = this.$slots).default) === null || _d === void 0 ? void 0 : _d.call(_c)) || (env_1.debug ? [emptyNode] : []);
        var rendered;
        if (this.rejected && this.$slots.error) {
            rendered = createWrapper(this.$slots.error);
        }
        else {
            rendered = isVisible
                ? createWrapper(this.resolved ? children : children.concat(fallback))
                : createWrapper([
                    // We need to render the children, but we should not show the rendered content.
                    createHiddenWrapper(children, this.resolved),
                    fallback,
                ]);
        }
        return rendered;
    },
});
function createWrapper(children) {
    if (!vue_demi_1.isVue2 && typeof children !== 'function') {
        children = function () { return children; };
    }
    return (0, vue_demi_1.h)(fragment_1.FragmentComponent, children);
}
function createHiddenWrapper(children, display) {
    if (!vue_demi_1.isVue2 && typeof children !== 'function') {
        children = function () { return children; };
    }
    return (0, vue_demi_1.h)(exports.suspenseOptions.hiddleWrapperTag, {
        style: { display: display ? 'block' : 'none' },
        class: { 'suspense-hidden-wrapper': true },
    }, children);
}
